@@ -52,7 +52,7 @@ PATH_PRIVSEP = pathlib.Path("/run/sshd")
 
 
 @dataclasses.dataclass(frozen=True)
-class Config(cfg_diag.ConfigDiag):
+class Config(cfg_diag.Config):
     """Runtime configuration for the remrun test tool."""
 
     prog: pathlib.Path
@@ -116,7 +116,8 @@ def parse_args() -> Config:
 def find_listening_port(cfg: Config) -> tuple[str, int]:
     """Find a port to listen on at a local address."""
     for (addr, family) in (("127.0.0.1", socket.AF_INET), ("::1", socket.AF_INET6)):
-        cfg.diag(f"Looking for a port to listen on at {addr}")
+        # pylint: disable=cell-var-from-loop
+        cfg.diag(lambda: f"Looking for a port to listen on at {addr}")
         for port in range(8086, 8200):
             lsock = socket.socket(
                 family=family, type=socket.SOCK_STREAM, proto=socket.IPPROTO_TCP
@@ -125,12 +126,12 @@ def find_listening_port(cfg: Config) -> tuple[str, int]:
             try:
                 lsock.bind((addr, port))
                 lsock.close()
-                cfg.diag(f"- got {addr}:{port}")
+                cfg.diag(lambda: f"- got {addr}:{port}")
                 return addr, port
             except socket.error as err:
-                cfg.diag(f"- could not bind to {addr}:{port}: {err}")
+                cfg.diag_(f"- could not bind to {addr}:{port}: {err}")
 
-        cfg.diag(f"- could not bind to any of the desired ports at {addr}")
+        cfg.diag(lambda: f"- could not bind to any of the desired ports at {addr}")
 
     sys.exit("Could not find a local address/port to listen on")
 
@@ -158,29 +159,29 @@ def create_ssh_config(
     cli_dir.mkdir(mode=0o700)
     srv_dir.mkdir(mode=0o700)
 
-    cfg.diag(f"Generating the SSH host key at {srv_key}")
+    cfg.diag(lambda: f"Generating the SSH host key at {srv_key}")
     subprocess.check_call(
         ["ssh-keygen", "-f", srv_key, "-t", "ed25519", "-N", ""], env=cfg.utf8_env
     )
 
-    cfg.diag(f"Generating the SSH client key at {cli_key}")
+    cfg.diag(lambda: f"Generating the SSH client key at {cli_key}")
     subprocess.check_call(
         ["ssh-keygen", "-f", cli_key, "-t", "ed25519", "-N", ""], env=cfg.utf8_env
     )
 
-    cfg.diag("Copying the client public key to the authorized keys file")
+    cfg.diag_("Copying the client public key to the authorized keys file")
     srv_authkeys.write_text(
         cli_key.with_suffix(".pub").read_text(encoding="UTF-8"), encoding="UTF-8"
     )
     srv_authkeys.chmod(0o600)
 
-    cfg.diag("Generating the client known hosts file")
+    cfg.diag_("Generating the client known hosts file")
     cli_known.write_text(
         addr + " " + srv_key.with_suffix(".pub").read_text(encoding="UTF-8"),
         encoding="UTF-8",
     )
 
-    cfg.diag("Generating the SSH client config file")
+    cfg.diag_("Generating the SSH client config file")
     cli_cfg.write_text(
         f"""
 Host *
@@ -206,7 +207,7 @@ VerifyHostKeyDNS no
         encoding="UTF-8",
     )
 
-    cfg.diag("Generating the SSH server config file")
+    cfg.diag_("Generating the SSH server config file")
     srv_cfg.write_text(
         f"""
 AllowUsers {username}
@@ -232,7 +233,7 @@ UseDNS no
     subprocess.check_call(["grep", "-Ere", "^", "."], cwd=tempd, env=cfg.utf8_env)
 
     if username == "root":
-        cfg.diag("Unlocking the root account, just in case")
+        cfg.diag_("Unlocking the root account, just in case")
         subprocess.check_call(["usermod", "-U", "root"], env=cfg.utf8_env)
 
     return SSHConfig(
@@ -250,7 +251,7 @@ def create_ssh_wrapper(cfg: Config, ssh_cfg: SSHConfig) -> Config:
     home_bin = ssh_cfg.home / "bin"
     home_bin.mkdir(mode=0o700)
 
-    cfg.diag("Determining the full path of the real SSH executable")
+    cfg.diag_("Determining the full path of the real SSH executable")
     lines = subprocess.check_output(
         ["sh", "-c", "command -v ssh"], encoding="UTF-8", env=cfg.utf8_env
     ).splitlines()
@@ -262,7 +263,7 @@ def create_ssh_wrapper(cfg: Config, ssh_cfg: SSHConfig) -> Config:
             f"Expected `command -v ssh` to point to an executable file, got {ssh_prog!r}"
         )
 
-    cfg.diag("Generating the SSH wrapper")
+    cfg.diag_("Generating the SSH wrapper")
     ssh_wrapper = home_bin / "ssh"
     ssh_wrapper.write_text(
         f"""#!/bin/sh
@@ -280,7 +281,7 @@ exec {shlex.quote(str(ssh_prog))} -F {shlex.quote(str(ssh_cfg.client_config))} "
     else:
         utf8_env["PATH"] = f"{home_bin}:{opath}"
 
-    cfg.diag("Checking that the SSH-specific environment is sane")
+    cfg.diag_("Checking that the SSH-specific environment is sane")
     lines = subprocess.check_output(
         ["sh", "-c", "command -v ssh"], encoding="UTF-8", env=utf8_env
     ).splitlines()
@@ -292,7 +293,7 @@ exec {shlex.quote(str(ssh_prog))} -F {shlex.quote(str(ssh_cfg.client_config))} "
 
 def check_ssh_connection(cfg: Config, ssh_cfg: SSHConfig) -> None:
     """Once the SSH server has been started, check that the client can connect to it."""
-    cfg.diag("Checking that our SSH client and server both work")
+    cfg.diag_("Checking that our SSH client and server both work")
     lines = subprocess.check_output(
         ["sh", "-c", f"ssh -- {ssh_cfg.addr} printenv SSH_CONNECTION"],
         encoding="UTF-8",
@@ -310,25 +311,27 @@ def start_sshd(cfg: Config, ssh_cfg: SSHConfig) -> Iterator[subprocess.Popen[str
     """Start an SSH server listening at the specified address and port."""
     proc = None
     try:
-        cfg.diag("Starting an SSH server")
+        cfg.diag_("Starting an SSH server")
         proc = subprocess.Popen(
             ["/usr/sbin/sshd", "-D", "-e", "-f", ssh_cfg.server_config],
             encoding="UTF-8",
             env=cfg.utf8_env,
         )
-        cfg.diag(f"- got SSH server process {proc.pid}")
+        cfg.diag(lambda: f"- got SSH server process {proc.pid}")  # type: ignore[union-attr]
         yield proc
     finally:
         if proc is not None:
             if not proc.poll():
-                cfg.diag("Killing the SSH server")
+                cfg.diag_("Killing the SSH server")
                 proc.kill()
-            cfg.diag(f"The SSH server is done, code {proc.wait()}")
+            cfg.diag(
+                lambda: f"The SSH server is done, code {proc.wait()}"  # type: ignore[union-attr]
+            )
 
 
 def create_test_script(cfg: Config, tempd: pathlib.Path) -> pathlib.Path:
     """Create the test script that runs printenv with some arguments."""
-    cfg.diag("Creating the test printenv script to run on the other side")
+    cfg.diag_("Creating the test printenv script to run on the other side")
     test_printenv = tempd / "test_printenv"
     test_printenv.write_text(
         """#!/bin/sh
@@ -344,7 +347,7 @@ printenv USER SSH_CONNECTION
 def test_remrun(cfg: Config, ssh_cfg: SSHConfig, test_printenv: pathlib.Path) -> None:
     """Run remrun a couple of times, examine its output."""
     assert cfg.test_prog is None
-    cfg.diag("Now running remrun with our client against our server")
+    cfg.diag_("Now running remrun with our client against our server")
     lines = subprocess.check_output(
         [cfg.prog, "--", ssh_cfg.addr, test_printenv],
         encoding="UTF-8",
@@ -386,7 +389,7 @@ def test_remrun(cfg: Config, ssh_cfg: SSHConfig, test_printenv: pathlib.Path) ->
 def test_prog(cfg: Config, ssh_cfg: SSHConfig) -> None:
     """Run the run-test.sh test suite within our environment."""
     assert cfg.test_prog is not None
-    cfg.diag(f"Running the {cfg.test_prog} testsuite")
+    cfg.diag(lambda: f"Running the {cfg.test_prog} testsuite")
     run_env = dict(cfg.utf8_env)
     run_env["REMRUN_TEST_HOSTSPEC"] = ssh_cfg.addr
     subprocess.check_call(["sh", "--", cfg.test_prog, cfg.prog], env=run_env)
@@ -395,17 +398,19 @@ def test_prog(cfg: Config, ssh_cfg: SSHConfig) -> None:
 def ensure_privsep_path(cfg: Config, tempd: pathlib.Path) -> bool:
     """Make sure the SSH server will be able to chroot into /run/sshd."""
     if not PATH_PRIVSEP.is_dir():
-        cfg.diag(f"Creating the {PATH_PRIVSEP} directory")
+        cfg.diag(lambda: f"Creating the {PATH_PRIVSEP} directory")
         os.makedirs(PATH_PRIVSEP, mode=0o755)
 
     if cfg.unpriv_account is not None:
-        cfg.diag(f"Getting information about the {cfg.unpriv_account!r} account")
+        cfg.diag(
+            lambda: f"Getting information about the {cfg.unpriv_account!r} account"
+        )
         try:
             acc_ent = pwd.getpwnam(cfg.unpriv_account)
         except KeyError:
             sys.exit(f"Unknown user account {cfg.unpriv_account!r}")
 
-        cfg.diag(f"Changing the ownership of {tempd} to {cfg.unpriv_account!r}")
+        cfg.diag(lambda: f"Changing the ownership of {tempd} to {cfg.unpriv_account!r}")
         try:
             os.chown(tempd, acc_ent.pw_uid, acc_ent.pw_gid)
         except OSError as err:
@@ -416,12 +421,14 @@ def ensure_privsep_path(cfg: Config, tempd: pathlib.Path) -> bool:
 
         child_pid = os.fork()
         if child_pid != 0:
-            cfg.diag(f"Skipping the tests, waiting for process {child_pid} to end")
+            cfg.diag(
+                lambda: f"Skipping the tests, waiting for process {child_pid} to end"
+            )
             res = os.waitpid(child_pid, 0)
-            cfg.diag(f"Process {child_pid} ended, code {res}")
+            cfg.diag(lambda: f"Process {child_pid} ended, code {res}")
             return False
 
-        cfg.diag(f"Trying to setuid() to {cfg.unpriv_account!r}")
+        cfg.diag(lambda: f"Trying to setuid() to {cfg.unpriv_account!r}")
         try:
             os.setgid(acc_ent.pw_gid)
             os.setuid(acc_ent.pw_uid)
@@ -442,7 +449,9 @@ def create_temp_dir(cfg: Config) -> Iterator[pathlib.Path]:
     try:
         tempd_obj = tempfile.mkdtemp(prefix="run_sshd_test.", dir=".")
         tempd = pathlib.Path(tempd_obj).absolute()
-        cfg.diag(f"Using {tempd} as a temporary directory, initial uid {initial_uid}")
+        cfg.diag(
+            lambda: f"Using {tempd} as a temporary directory, initial uid {initial_uid}"
+        )
         yield tempd
     finally:
         if tempd_obj is not None:
@@ -450,11 +459,11 @@ def create_temp_dir(cfg: Config) -> Iterator[pathlib.Path]:
             current_uid = os.getuid()
             if initial_uid != current_uid:
                 cfg.diag(
-                    f"Not removing {tempd} in process {current_pid}: "
+                    lambda: f"Not removing {tempd} in process {current_pid}: "
                     f"uid {current_uid} != {initial_uid}"
                 )
             else:
-                cfg.diag(f"Removing {tempd} in process {current_pid}")
+                cfg.diag(lambda: f"Removing {tempd} in process {current_pid}")
                 shutil.rmtree(tempd)
 
 
@@ -472,13 +481,13 @@ def wait_for_sshd_banner(cfg: Config, ssh_cfg: SSHConfig) -> None:
     s_family, s_type, s_proto = ainfo[:3]
     s_address, s_port = ainfo[4][:2]
     cfg.diag(
-        f"Waiting for the SSH server at {s_address}:{s_port} "
+        lambda: f"Waiting for the SSH server at {s_address}:{s_port} "
         f"(address family {s_family.name}) to start accepting connections"
     )
 
     for _ in range(20):
         time.sleep(0.5)
-        cfg.diag(f"Trying to connect to {s_address} port {s_port}...")
+        cfg.diag(lambda: f"Trying to connect to {s_address} port {s_port}...")
         with socket.socket(s_family, s_type, s_proto) as sock:
             try:
                 sock.connect((s_address, s_port))
@@ -488,9 +497,10 @@ def wait_for_sshd_banner(cfg: Config, ssh_cfg: SSHConfig) -> None:
                 print("Connection refused, will retry")
                 continue
 
-            cfg.diag("Connected!")
+            cfg.diag_("Connected!")
             data = sock.recv(4096).decode("ISO-8859-15")
-            cfg.diag(f"Got banner {data!r}")
+            # pylint: disable-next=cell-var-from-loop
+            cfg.diag(lambda: f"Got banner {data!r}")
             if not data.startswith("SSH-") or "\n" not in data:
                 sys.exit(
                     f"Expected an SSH banner from the server started at {s_address}:{s_port}, "
